﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Linq.Expressions;

namespace LambdaExpressions {
  class Program {
    static void Main(string[] args) {
      // This approach uses a specific delegate type as well as 
      // specially prepared methods to pass in.
      DoSomething(Return42);
      DoSomething(Add10);

      // C# 2.0 allows us to pass in anonymous methods instead:
      DoSomething(delegate(int input) {
        return input * input;
      });

      // Finally, in C# 3.0, we can replace the whole thing with a 
      // lambda expression:
      DoSomething(x => x * x * x);

      // These following lines should duplicate the output from above,
      // but now we're using a standard generic delegate type instead 
      // of our own.
      DoSomethingInVersion3(Return42);
      DoSomethingInVersion3(Add10);
      DoSomethingInVersion3(delegate(int input) {
        return input * input;
      });
      DoSomethingInVersion3(x => x * x * x);

      // There are several Func<...> delegates available with varying
      // numbers of parameters, and lambda expressions can take 
      // various different formats from very verbose to very concise.
      Func<int, int, bool> calculation =
        (int val1, int val2) => {
          if (val1 + val2 == 42)
            return true;
          else
            return false;
        };
      Func<int, int, bool> sameCalculation = (v1, v2) => v1 + v2 == 42;

      // A lambda expression with an expression body can be stored 
      // as an expression instead of a delegate.
      Expression<Func<int, int, bool>> exp1 = (v1, v2) => v1 + v2 == 42;

      // The same thing doesn't work for a lambda expression with a 
      // statement body. This will fail at compile time.
      Expression<Func<int, int, bool>> exp2 =
        (int val1, int val2) => {
          if (val1 + val2 == 42)
            return true;
          else
            return false;
        };
    }

    static void DoSomething(TakeIntReturnIntDelegate function) {
      Console.WriteLine(function(10));
    }

    static void DoSomethingInVersion3(Func<int, int> function) {
      Console.WriteLine(function(10));
    }
    
    delegate int TakeIntReturnIntDelegate(int input);

    static int Return42(int input) {
      return 42;
    }

    static int Add10(int input) {
      return input + 10;
    }
  }
}
